clearvars;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%                              path + data                                %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
data.hor    = 20;
data.init   = 100;
data.long   = 192;
data.nsim   = 250;

rng(1234567890,'twister')
data.e1     = randn(data.nsim,data.long+data.init);
data.e2     = randn(data.nsim,data.long+data.init);
ficm        = 'real_uexpost_h1st';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%                      Parameters + Initial Conditions                    %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

strmom  = [];
optim_  = 0;

ra1     = 1-1e-12;      % Random walk for technology shock
rs1     = 0.75;         % Autocorrelation of belief shock
whsy    = 0.7;          % labor share
mkup    = 0.0;          % No markup
mcss    = 1/(1+mkup);
thx     = (1+mkup)/mkup;
alpha   = 1-whsy/mcss;
beta    = 0.99;         % Discount factor
delta   = 0.025;        % Depreciation rate
sba     = 0;            % No correlation between innovations    
sigc    = 1;            % IELS
rho     = 1;            % Degree of Complementarity
sigh    = 0.5;          % inverse Frish elasticity
sigz    = 1;            % Utilization (1->no utilization)
ses     = 5.72;         % sets to match std(y,c,i,h), actually innocuous for the results
sea     = 0;            % shuts down the technology shock
%
% Choice of models:
% =================
Names   = {'ses','sea','alpha','mkup','beta','delta','sigc','sigh','sigz','rho','ra1','rs1','sba'};
Param   = [ses;sea;alpha;mkup;beta;delta;sigc;sigh;sigz;rho;ra1;rs1;sba];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%                              Model Output                               %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sol                 = feval(ficm,Param,Names);
[strtmp,irfa,irfs]  = model_output_benchmark(Param,ficm,Names);
strmom              = strcat(strmom,strtmp);
disp('Moments (BP-filtered)')
disp(strmom)

%% IRFs to a sentiment shock that shifts output up by 1 %
fs      = 12;       % fontsize
blue    = [0 0 1];  % color blue
sel     = [1 2 3 4 5];
ylim    = [0 1.05;0 0.5;-0.25 4.25;-0.1 1.5;-1 1];
str     = {'Output','Consumption','Investment','Hours Worked','Productivity'};
figure('name','IRFs to a sentiment shock that shifts output up by 1%')
for i=1:5
    subplot(3,5,i);
    h=plot(irfs.y(:,sel(i))/irfs.y(1,sel(1)));
    set(h(1),'linestyle','-','linewidth',2,'color',blue)
    set(gca,'fontname','times','fontsize',10,'ylim',ylim(i,:))
    title(str{i},'fontname','times','fontsize',10,'fontweight','normal')
    xlabel('Quarters','fontname','times','fontsize',10,'fontweight','normal')
    if i==1
        ylabel('% deviation','fontname','times','fontsize',10,'fontweight','normal')
    end
end
%% Forecasts of terms of trade, following a confidence shock
figure('name','Forecasts of terms of trade, following a confidence shock')
term=termstruc(sol,12,rho,1);
subplot(221)
h=plot(0:12,ses*term([1 4],:)');
set(h(1),'linestyle','-','linewidth',2,'color',blue)
set(h(2),'linestyle','--','linewidth',2,'color',blue)
set(gca,'fontname','times','fontsize',12,'xlim',[1 12])
xlabel('Forecast Horizon in Quarters','fontname','times','fontsize',fs);
o=legend('On impact','One year later','location','northeast');
set(o,'fontname','times','fontsize',12)

