function sol=solve_beliefs(model,varargin)
if nargin>1
    nforecasts=varargin{1};
else
    nforecasts=5;
end

% Measurement equation

Myy = model.myy;
Myx = model.myx;
MyX = model.myX;
MyY = model.myY;
Myf = model.myf;
MyF = model.myF;
Mys = model.mys;

% Backward looking part

Mxx0= model.mxx0;
Mxx1= model.mxx1;
MxX1= model.mxX1;
Mxy1= model.mxy1;
MxY1= model.mxY1;
Mxf1= model.mxf1;
MxF1= model.mxF1;
Mxs1= model.mxs1;
nb  = size(model.mxx0,1);
% Forward looking part

Mff0= model.mff0;
MfF0= model.mfF0;
Mff1= model.mff1;
MfF1= model.mfF1;
Mfx0= model.mfx0;
Mfx1= model.mfx1;
MfX1= model.mfX1;
Mfy0= model.mfy0;
MfY0= model.mfY0;
Mfy1= model.mfy1;
MfY1= model.mfY1;
Mfs0= model.mfs0;
Mfs1= model.mfs1;
nf  = size(model.mff0,1);
% Shocks
R   = model.r;
ns  = size(R,1);
Q   = model.q;
nx  = size(Q,1);
DELT= model.Delta;

% Trsnformation of matrices

Mx  = Myy\Myx;
MX  = Myy\(Myx+MyX);
MEY = Myy\MyY;
Mf  = Myy\Myf;
MF  = Myy\(Myf+MyF);
Ms  = Myy\Mys;
I   = eye(size(MEY,1));

Nx  = Mxx0\Mxx1;
NX  = Mxx0\(Mxx1+MxX1);
Ny  = Mxx0\Mxy1;
NY  = Mxx0\(Mxy1+MxY1);
Nf  = Mxx0\Mxf1;
NF  = Mxx0\(Mxf1+MxF1);
Ns  = Mxx0\Mxs1;

Pf0 = Mff0;
Pf1 = Mff1+Mfx0*Nf;
PF0 = MfF0;
PF1 = Mff1+MfF1+Mfx0*NF;
Px  = Mfx1+Mfx0*Nx;
PX  = Mfx1+MfX1+Mfx0*NX;
Py0 = Mfy0;
PY0 = Mfy0+MfY0;
Py1 = Mfy1+Mfx0*Ny;
PY1 = Mfy1+MfY1+Mfx0*NY;
Ps  = Mfs0*R+Mfs1+Mfx0*Ns;

% STEP 1: Solve the representative agent model

M0  = [ eye(nb) zeros(nb,nf);
    -PY0*((I-MEY)\MX)  Pf0-PF0-PY0*((I-MEY)\MF)];

M1  = [ NX+NY*((I-MEY)\MX) NF+NY*((I-MEY)\MF)
    PX+PY1*((I-MEY)\MX) PF1+PY1*((I-MEY)\MF)];

M2  = [Ns+NY*((I-MEY)\Ms);Ps+PY1*((I-MEY)\Ms)+PY0*((I-MEY)\Ms)*R];

[s,t,q,z]   = qz(M0,M1);            % Generalized Eigenvalues
[s,t,~,z]   = ordqz(s,t,q,z,'udo'); % Reorders eigenvalues

z21     = z(nb+1:nb+nf,1:nb);
z11     = z(1:nb,1:nb);

if rank(z11)<nb;
    warning('Representative Agent problem: Invertibility condition violated')
    sol.id  = -1;
else
    z11i    = z11\eye(nb);
    s11     = s(1:nb,1:nb);
    t11     = t(1:nb,1:nb);
    
    if or(abs(t(nb,nb))>abs(s(nb,nb)),abs(t(nb+1,nb+1))<abs(s(nb+1,nb+1)))
        warning('Representative Agent problem: Wrong number of stable eigenvalues.');
        sol.id=1;
    else
        
        dyn     = s11\t11;
        GAMfX      = z21*z11i;
        if max(max(imag(GAMfX)))<1e-6;
            GAMfX = real(GAMfX);
        end
        OMX    = z11*dyn*z11i;
        if max(max(imag(OMX)))<1e-6;
            OMX  = real(OMX);
        end
        B       = 1:nb;
        F       = nb+1:nb+nf;
        
        PSI0    =-(M0(B,B)+M0(B,F)*GAMfX)\M0(B,F);
        PSI1    = (M0(B,B)+M0(B,F)*GAMfX)\M1(B,F);
        PSI2    = (M0(B,B)+M0(B,F)*GAMfX)\M2(B,:);
        G0      = (M0(F,B)+M0(F,F)*GAMfX)*PSI0+M0(F,F);
        G1      = (M0(F,B)+M0(F,F)*GAMfX)*PSI1-M1(F,F);
        G2      = M2(F,:)-(M0(F,B)+M0(F,F)*GAMfX)*PSI2;
        Qtmp    = (kron(R',G0)+kron(eye(ns),G1));
        
        tmp     = Qtmp\G2(:);
        GAMfS   = reshape(tmp,nf,ns);
        OMS     = PSI0*GAMfS*R+PSI1*GAMfS+PSI2;
        LByX    = (I-MEY)\(MX+MF*GAMfX);
        LByz    = (I-MEY)\(Ms+MF*GAMfS);
        sol.id  = 0;
        
    end
end

if sol.id==0
    % STEP 2: Solve the individual problem
    M0  = [ eye(nb) zeros(nb,nf);
        -Py0*Mx  Pf0-Py0*Mf];
    
    M1  = [ Nx+Ny*Mx Nf+Ny*Mf
        Px+Py1*Mx Pf1+Py1*Mf];
    
    [s,t,q,z]   = qz(M0,M1);            % Generalized Eigenvalues
    [s,t,~,z]   = ordqz(s,t,q,z,'udo'); % Reorders eigenvalues
    
    z21     = z(nb+1:nb+nf,1:nb);
    z11     = z(1:nb,1:nb);
    
    if rank(z11)<nb;
        warning('Individual''s Problem: Invertibility condition violated')
        sol.id  = -1;
    else
        z11i    = z11\eye(nb);
        s11     = s(1:nb,1:nb);
        t11     = t(1:nb,1:nb);
        
        if or(abs(t(nb,nb))>abs(s(nb,nb)),abs(t(nb+1,nb+1))<abs(s(nb+1,nb+1)))
            warning('Individual''s Problem:Wrong number of stable eigenvalues.');
            sol.id=1;
        else
            
            dyn     = s11\t11;
            GAMfx   = z21*z11i;
            if max(max(imag(GAMfx)))<1e-6;
                GAMfx   = real(GAMfx);
            end
            OMx    = z11*dyn*z11i;
            if max(max(imag(OMx)))<1e-6;
                OMx  = real(OMx);
            end
            LByx    = Mx+Mf*GAMfx;
            sol.id  = 0;
        end
    end
end
% Step 3: Solving for GAMfz, GAMfzb, GAMfs
% GAMfs
if sol.id==0
    As=-(((PF0-Pf0)*GAMfX+PY0*LByX)*NF+PF1);
    Bs=Ps+PY0*(LByz*R+LByX*Ns)+(PF0-Pf0)*(GAMfS*R+GAMfX*Ns);
    GAMfs=As\Bs;
    OMs = Ns+NF*GAMfs;
    
    % GAMfz
    Az=(Pf0*GAMfx-Py0*LByx)*Nf-Pf1;
    Bz=Py1*LByz-(Pf0*GAMfx-Py0*LByx)*Ny*LByz;
    GAMfz=Az\Bz;
    OMz=Ny*LByz+Nf*GAMfz;
    
    % GAMfzb
    GAMfzb=GAMfS-GAMfz-GAMfs;
    OMzb=(NY-Ny)*LByz+(NF-Nf)*GAMfz+NF*GAMfzb;
    
    % Uncomment the following to check that restrictions are satisfied
    % disp('Condition on GAMfz')
    % resz= Pf0*GAMfx*OMz-(Pf1*GAMfz+Py0*LByx*OMz+Py1*LByz);
    % disp(resz)
    % disp('Condition on GAMfzb')
    % reszb=(Pf0-PF0)*((GAMfX-GAMfx)*OMz+GAMfX*OMzb)-(PF0*GAMfx*(Ny*LByz+Nf*GAMfz)+(PF1-Pf1)*GAMfz+PF1*GAMfzb+PY0*LByX*(OMz+OMzb)-Py0*LByx*OMz+(PY1-Py1)*LByz);
    % disp(reszb)
    % disp('Condition on GAMfs')
    % ress=(Pf0-PF0)*(GAMfX*OMs+GAMfS*R)-(PF1*GAMfs+PY0*(LByX*OMs+LByz*R)+Ps);
    % disp(ress)
    % Step 4: Solving for GAMfxi
    % A GFxi Q +B GFxi = C
    A   = (PF0-Pf0)+PY0*((I-MEY)\MF);
    B   = ((PF0-Pf0)*GAMfX*NY+PY0*LByX*NY+PY1)*((I-MEY)\MF)+(PF0-Pf0)*GAMfX*NF+PF1+PY0*LByX*NF;
    C0  = (PF0*GAMfz+(PY0-Py0)*LByz+(PF0-Pf0)*GAMfzb+PY0*((I-MEY)\(MF*GAMfzb+(MF-Mf)*GAMfz+MEY*LByz)))*DELT*Q;
    C1  = ((PF0-Pf0)*GAMfX*NY+PY0*LByX*NY+PY1)*((I-MEY)\(MF*GAMfzb+(MF-Mf)*GAMfz+MEY*LByz))*DELT;
    C   = C0+C1;
    tmp = kron(Q',A)+kron(eye(nx),B);
    
    vGxi    = -tmp\C(:);
    GAMfxi  = reshape(vGxi,nf,nx);
    LByxi   = (I-MEY)\(MF*(GAMfxi+GAMfzb*DELT)+(MF-Mf)*GAMfz*DELT+MEY*LByz*DELT);
    OMxi    = NF*GAMfxi+NY*LByxi;
    
    % Uncomment the following to check that restrictions is satisfied
    % disp('Condition on GAMfxi')
    % resxi=(PF0-Pf0)*(GAMfX*OMxi+(GAMfxi+GAMfzb*DELT)*Q)+PF1*GAMfxi+PY0*(LByX*OMxi+LByxi*Q)+(PY0-Py0)*LByz*DELT*Q+PY1*LByxi;
    % disp(resxi)
    sol.mx  = [OMX OMS OMxi;zeros(ns,nb) R zeros(ns,nx);zeros(nx,nb+ns) Q];
    sol.me  = [zeros(nb,ns+nx);eye(ns+nx)];
    sol.my  = [LByX LByz LByxi;GAMfX GAMfS GAMfxi];
    sol.mxx = OMX;
    sol.mxs = OMS;
    sol.mxxi= OMxi;
    sol.myx = [LByX;GAMfX];
    sol.mys = [LByz;GAMfS];
    sol.myxi= [LByxi;GAMfxi];
    sol.gamfX=GAMfX;
    sol.gamfx=GAMfx;
    sol.gamfxi=GAMfxi;
    sol.gamfs=GAMfs;
    sol.gamfz=GAMfz;
    sol.gamfzb=GAMfzb;
    sol.omX=OMX;
    sol.omx=OMx;
    sol.omxi=OMxi;
    sol.oms=OMs;
    sol.omz=OMz;
    sol.omzb=OMzb;
    sol.lbzb=LByz;
    sol.lbx=LByxi;
    sol.r   = R;
    sol.q   = Q;
    sol.delt= DELT;
    sol.err = [LByz;GAMfS]*DELT;
else
    sol.id=-1;
    sol.mx  = [];
    sol.my  = [];
    sol.me  = [];
    sol.sig = [];
end