clear
% function main_nkms_hpiac
addpath('method');
addpath('transfo');
% addpath('models');
addpath('extras');
warning off
%%
%
% some file names
%
model.name      = 'real_model_utilization_ci_timing';
fileres         = 'res_real_util_ci';
fcase           = 'real_util_ci';
% model.transfo   = 'transfo_nk_utilization';
filepriors      = 'priors/datapriors_real_utilization.txt';

ishocks         = 4:11;
nom_            = 0;
model.type      = 'fd';        % fd for Freq. Domain, td for Time Domain
model.filter    = 'bp';
model.wmin      = 2*pi/32;
model.wmax      = 2*pi/6;
model.hp        = 1600;
model.select    = 1:4;          % Selection of variables in MY matrix
model.ish       = 1:8;
model.shstr     = {
    'Technology shock (Permanent)','Technology shock (News)','Technology shock (Transitory)',...    
    'Investment shock (Permanent)','Investment shock (Transitory)',...
    'Discount Shock','Government shock','Belief shock'};
model.sh        = {'ap','an','at','ip','it','c','g','xi'};
folder          = 'results';
filedata        = 'data/data_for_estimation';
if ~exist(folder,'dir')
    mkdir(folder)
end
file.res        = strcat(folder,'/res_',fcase,'.txt');	% File name for results
file.mode       = strcat(folder,'/mode_',fcase);        % File name for housekeeping of mode information
file.metro      = strcat(folder,'/metro_',fcase);       % File name for housekeeping of metropolis draws
optimizer_      = 'gn';                                 % 1 to perform optimization, 0 to load previous mode result
save_           = 1;                                    % 1 to save results, 0 otherwise
optim_          = 1;                                    % 1 to perform optimization, 0 to load previous mode result
hessian_        = 1;                                    % 1 to compute Hessian
metro_          = 1;                                    % 1 to perform posterior simulation using MH, 0 to load previous draws
eval_           = 1;
atmode_         = 0;
%% Data
load(filedata);
whole   = 1960:0.25:2011.75;
T0      = 1960;
T1      = 2007.75;
smpl    = find(whole==T0):find(whole==T1);
hp      = hpmat(length(smpl),model.hp);
order   = 1;
X       = [ly lc li];
dataest = 100*detrend(X(smpl,:),order);
% tmp     = 100*detrend([lhnfb(smpl) dp(smpl) ffr(smpl)],0);
tmp     = 100*detrend([lhnfb(smpl)],0);
dataest = [dataest(:,1:3) tmp ];
datares = dataest;
%%
c               = 0.4;
%   
% Specify the priors
%
fid             = fopen(filepriors);
tmp             = textscan(fid,'%s%s%s%f%f%f%f%f');
fclose(fid);
latexnames      = tmp{1};
name            = tmp{2};
datapriors.type = tmp{3};
datapriors.mean = tmp{4};
datapriors.std  = tmp{5};
lwbound         = tmp{6};
upbound         = tmp{7};
guess=[
    0.7    % \rho_{a}
    0.7     % \rho_{n}
    0.7     % \rho_{i}
    0.7     % \rho_{c}
    0.7     % \rho_{g}
    0.7     % \rho_{x}
    0.5       % \sigma_ap
    0.5       % \sigma_at
    0.5       % \sigma_an
    0.5       % \sigma_ip
    1         % \sigma_it
    0.5       % \sigma_c
    1         % \sigma_g
    0.5       % \sigma_\xi
    0.5     % \sigma_z
    0.5     % \sigma_h
    0.3     % \alpha
    2       % \phi
    0.5     % b
    ];

% guess=datapriors.mean;
if isfield(model,'transfo');
    theta0	= feval(strcat('inv',model.transfo),guess);
else
    theta0  = guess;
end

itheta  = 1:length(theta0);
ifixed  = [];
fixed   = [];
itheta  = setdiff(itheta,ifixed);
theta0  = theta0(itheta);

clear tmp
bounds          = [lwbound(:) upbound(:)];
k               = 1;
nparest         = length(itheta);
tmp             = cell(nparest,1);
for i=itheta
    tmp{k}      = name{i};
    k           = k+1;
end
name            = tmp;

% MH Algorithm
%-------------
nchain          = 2;                % # of chains
latex           = 1;                % set to 1 to print out results in LaTeX form
NDraws          = 200000;           % number of draws for MH-algoritm
discard         = ceil(0.1*NDraws); % # of draws to be discarded
step            = 1;                % keep draws every step draws
metro.ndraws    = NDraws;
metro.discard   = discard;
metro.step      = step;
metro.c0        = 1;                % initial rejection rate
metro.c         = c;                % rejection rate
% Initialize Matrices
%---------------------
ncoef           = size(theta0,1);
% Compute Posterior mode and inverse hessian matrix
%---------------------------------------------------
opts.crit       = 1e-5;
opts.nit        = 1e9;
opts.verb       = 1;
opts.house      = 0;
opts.display    = 1;
switch optim_
    case 1
        switch optimizer_
            case 'cm'   % CMAES algorithm
%                 optss           = optimset('display','iter','maxiter',1e6,'maxfunevals',1e6);
                optss.LBounds=lwbound(:);
                optss.UBounds=upbound(:);
                xx              = fab_cmaes(@objective,theta0,datapriors.std,optss,dataest,model,datapriors,bounds,fixed,itheta,ifixed,datares);
                if isfield(model,'transfo')
                    theta_mode  = feval(model.transfo,xx);
                else
                    theta_mode  = xx;
                end
            case 'nm'   % Nelder-Meade algorithm
                optss           = optimset('display','iter','maxiter',1e6,'maxfunevals',1e6);
                xx              = fminsearch(@objective,theta0,optss,dataest,model,datapriors,bounds,fixed,itheta,ifixed,datares);
                if isfield(model,'transfo')
                    theta_mode  = feval(model.transfo,xx);
                else
                    theta_mode  = xx;
                end
            case 'gn'   % Gauss Newton (C. Sims) algorithm
                h0              = 1e-5*eye(ncoef);
                sol             = fcmin('objective',theta0,h0,[],opts,dataest,model,datapriors,bounds,fixed,itheta,ifixed,datares);
                iH              = sol.ihess;
                if isfield(model,'transfo')
                    theta_mode  = feval(model.transfo,sol.x);
                else
                    theta_mode  = sol.x;
                end
            case 'sa'   % Simulated Annealing
                opts.CoolSched  = @(T) (0.95*T);
                opts.InitTemp   = 1;
                opts.StopTemp   = 1e-12;
                opts.MaxTries   = 1000;
                opts.MaxSuccess = 25;
                xx              = anneal(@objective,theta0',opts,dataest,model,datapriors,bounds,fixed,itheta,ifixed,datares);
                if isfield(model,'transfo')
                    theta_mode  = feval(model.transfo,xx);
                else
                    theta_mode  = xx;
                end
        end
        theta_mode  = theta_mode(:);
        [loglike,~,loglh,jointprior]=evalpost(theta_mode,dataest,model,datapriors,bounds);
        fprintf('Posterior mode: %12.8f\n',loglike);
        fprintf('    Prior mode: %12.8f\n',jointprior);
        fprintf('  Std. Loglike: %12.8f\n',loglh);
        fprintf('Posterior mode: \n');
        for i=1:length(theta_mode)
            fprintf('%8s: %g \n',name{i},theta_mode(i));
        end
        sol=feval(model.name,theta_mode);
        mom=comp_mombp(sol,[1:4]);
        disp(mom.std)
        disp(mom.cor)
        [VD,CY,CH,CI]= vdecfreq(sol,model.wmin,model.wmax,[1 2 3 4]);
        disp(VD')
        switch hessian_
            case 1
                [hess,ihess]    = hessian(theta_mode,dataest,model,datapriors,bounds,fixed,itheta,ifixed);
                if save_
                    delete(strcat(file.mode,'.mat'))
                    save(strcat(file.mode,'.mat'),'theta_mode','loglike','hess','ihess')
                end
            otherwise
                if save_
                    delete(strcat(file.mode,'.mat'))
                    save(strcat(file.mode,'.mat'),'theta_mode','loglike')
                end
        end
        
    otherwise
        load(strcat(file.mode))
end
%%


% return
%%
%
% Metropolis-Hastings
% --------------------
%

TH      = zeros(nchain*NDraws,nparest);
PTH     = zeros(nchain*NDraws,1);
switch metro_
    case 1
        for i=1:nchain
            fprintf('\nGenerating Markov Chain #%2d\n',i)
            [theta,p_theta]=metro_hast(theta_mode,ihess,dataest,datapriors,model,metro,bounds,fixed,itheta,ifixed);
            if save_
                eval(['save ' file.metro int2str(i) '.mat theta p_theta'])
            end
            TH((i-1)*NDraws+1:i*NDraws,:)   = theta;
            PTH((i-1)*NDraws+1:i*NDraws,:)  = p_theta;
        end
    case -1
        disp('Skip MCMC')
    otherwise
            for i=1:nchain
                eval(['load ' file.metro int2str(i) '.mat'])
                TH((i-1)*NDraws+1:i*NDraws,:)   = theta;
                PTH((i-1)*NDraws+1:i*NDraws,:)  = p_theta;
            end
end
return
step    = 10;
%%
TH      = zeros(nchain*NDraws,nparest);

            for i=1:nchain
                eval(['load ' file.metro int2str(i) '.mat'])
                TH((i-1)*NDraws+1:i*NDraws,:)   = theta;
            end

%% EVALUATION OF DRAWS
%---------------------
%
%
if eval_
    fid     = fopen(strcat(file.res),'a');
    stats   = disp_results(TH,PTH,theta_mode,step,latexnames,latex,fid);
    fclose('all');
end
% return
%%
if~isempty(ifixed)
    switch atmode_
        case 1
        otherwise
            TH0     = TH;
            TH      = zeros(size(theta_mode,1),length(itheta)+length(ifixed));
            TH(:,itheta)= TH0;
            TH(:,ifixed)= repmat(fixed,length(theta_mode),1);
            clear TH0;
    end
else
    if atmode_
        TH      = theta_mode;
    end
end
%%
atmode_=0;
if atmode_
    TH      = theta_mode;
else
    TH      = mean(TH(1:step:end,:));
end
%%

irf=irf_model_mode(TH,model,strcat('irf_',fcase),nom_);
%%
% vdec_beliefs_model_mode(TH,model,nom_);
sol     = feval(model.name,TH);
ne      = size(sol.me,2);
solk    = sol;
solk.my = solk.my(model.select,:);
ks      = kalman_smoother(datares,solk);
ymod    = (sol.my*ks.x')';
idx=[4 11];
model.level=1;
graph_shocks_mode(model,dataest,TH,strcat('shocks_',fcase),idx);
%%
[VD,VDr,VDTr,CY,CH,CI]= vdecfreq(sol,model.wmin,model.wmax,[1 2 3 4 5]);
VD=VD';
CY      = CY';
CH      = CH';
CI      = CI';
for i=1:ne
    fprintf(strcat(repmat('& %6.2f ',1,5),'\\\\ \n'),VD(i,:))
end

%
for i=1:ne
    fprintf(strcat(repmat('& %6.2f ',1,6),'\\\\ \n'),100*[CY(i,[4 3 2]) CH(i,[3 2]) CI(i,2)])
end


%%
select  = [1 3 4 2 5];
mom=comp_mombp(sol,select);

for i=1:length(select);
    fprintf('& %6.2f \n',mom.std(i))
end
disp(' ')
for i=2:length(select);
    fprintf('& %6.2f \n',mom.cor(i,1))
end
disp(' ')
for i=[2 3 5];
    fprintf('& %6.2f \n',mom.cor(i,4))
end
% save(strcat('results/',fileres),'TH','irf','sol','ks');
%
% IDX={[3 4],8};%{[3 4],8,[1],[2],[5],[6],[7]};
% [Yt3,SVA3,SVT3,SVTr3,SVxi3,IRFP13,IRFP23]=nkmodel_vs_var3(sol,1,IDX,8);
% save(strcat('results/',fileres),'TH','irf','sol','ks','mom','Yt3','SVA3','SVT3','SVTr3','SVxi3','VD','VDr','VDTr');
% [Yt3,SVA3,SVT3]=nkmodel_vs_varc(sol,1,IDX,9);
save(strcat('results/',fileres),'TH','irf','sol','ks','mom','VD','VDr','VDTr','CH','CY','CI');
%%
%for j=1:9
%for i=1:7;fprintf(strcat(repmat('%6.2f  ',1,8),'\n'),SVxi3{i}(j,:));end;disp(' ');end
%%
