function stats=disp_priors(theta,name,latex);

if nargin<3
    latex=0;
end

% posterior moments
% -------------------
[sdraws,ncoef]  = size(theta);                      % length of the simulated estimates
nwlags          = 100;                              % # of lags for NW

theta_ci5       = prctile(theta,[2.5 97.5])';       % 5% confidence interval
theta_ci10      = prctile(theta,[5 95])';           % 10% confidence interval
theta_mean      = mean(theta);                      % mean
theta_std       = std(theta);                       % std
theta_nwstd     = sqrt(diag(nwvar(theta,nwlags)));  % NW std
theta_median    = median(theta);                    % median
theta_var       = cov(theta);                       % covariance matrix

stats.mean      = theta_mean;
stats.median    = theta_median;
stats.std       = theta_std;
stats.nwstd     = theta_nwstd;
stats.ci5       = theta_ci5;
stats.ci10      = theta_ci10;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                          %
%                           Displays Main results                          %
%                                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if latex
disp('\begin{table}[htbp]')
disp('\caption{Posterior Results}\label{tab:}')
disp('$$')
disp('\begin{tabular}{lccccc}')
disp('\hline')
disp('\hline')
disp('Param. & Mean & Median & Std. Dev. & 95\% Conf. Int. \\')
disp('\hline')
for i=1:ncoef;
    fprintf('%12s & %8.4f & %8.4f & %8.4f & [%8.4f,%8.4f] \\\\ \n',name{i},theta_mean(i),theta_median(i),theta_std(i),theta_ci10(i,1),theta_ci10(i,2))
end
disp('\hline')
disp('\hline')
disp('\end{tabular}')   
disp('$$')
disp('\end{table}')   
else
disp('-------------------------------------------------------------------------------------')
disp('       Param        Mean      Median       Stdev        5%-L        5%-H')
disp('-------------------------------------------------------------------------------------')
for i=1:ncoef;
    fprintf('%12s\t%8.3f\t%8.3f\t%8.3f\t%8.3f\t%8.3f\n',name{i},theta_mean(i),theta_median(i),theta_std(i),theta_ci10(i,1),theta_ci10(i,2))
end
disp('-------------------------------------------------------------------------------------')
end