function stats=disp_results(theta,p_theta,theta_mode,step,name,latex,varargin)

if nargin<5
    latex=0;
end

% posterior moments
% -------------------
[sdraws,nth]    = size(theta);                      % length of the simulated estimates
nwlags          = 100;                              % # of lags for NW
theta           = theta(1:step:sdraws,:);
p_theta         = p_theta(1:step:sdraws,:);
[sdraws,ncoef]  = size(theta);                      % length of the simulated estimates

theta_ci5       = hpdi(theta,95)';
theta_ci10      = hpdi(theta,90)';
% theta_ci5       = prctile(theta,[2.5 97.5])';       % 5% confidence interval
% theta_ci10      = prctile(theta,[5 95])';           % 10% confidence interval
theta_mean      = mean(theta);                      % mean
theta_std       = std(theta);                       % std
theta_nwstd     = sqrt(diag(nwvar(theta,nwlags)));  % NW std
theta_median    = median(theta);                    % median
theta_var       = cov(theta);                       % covariance matrix

stats.mean      = theta_mean;
stats.mode      = theta_mode;
stats.median    = theta_median;
stats.std       = theta_std;
stats.nwstd     = theta_nwstd;
stats.ci5       = theta_ci5;
stats.ci10      = theta_ci10;

tolvar          = 1e-6;
i               = find(diag(theta_var)<tolvar);
theta_var(i,i)  = tolvar;
[L,U]           = lu(theta_var);    % LU representation of theta_var
ITV             = inv(U)*inv(L);    % Accurate computation of inverse of Sigma
tic
% posterior marginal densities obtained by
% Geweke Harmonic Mean Estimator
%------------------------------
p               = (0.1:0.1:0.9)';   % truncation parameter
pcrit           = chis_inv(p,ncoef);
np              = size(p,1);
hmax            = 2;
suminvlike      = zeros(np,1);
laginvlike      = zeros(hmax,np);
gaminvlike      = zeros(hmax,np);

% truncated multivariate normal distribution for harmonic mean (geweke) estimate of marginal density
mp_theta        = mean(p_theta);
tmp             = -log(p)-0.5*nth*log(2*pi)-0.5*log(det(theta_var));
for i=1:sdraws;
    tmp_th      = ((theta(i,:)-theta_mean)*ITV*(theta(i,:)-theta_mean)');
    lnf         = tmp-0.5*tmp_th;
    ind         = tmp_th<pcrit;     % truncation index
    invlike     = exp(lnf-(p_theta(i)-mp_theta)).*ind;
    laginvlike  = [invlike';laginvlike(1:hmax-1,:)];
    gaminvlike  = gaminvlike+laginvlike.*repmat(invlike',hmax,1);
    suminvlike  = suminvlike+invlike;
end
meaninvlike = suminvlike/sdraws;
gaminvlike  = gaminvlike/sdraws-repmat((meaninvlike.^2)',hmax,1);
marg_data_dens  = mp_theta-log(meaninvlike);
% Compute Standard Errors
suminvlikeerror = gaminvlike(1,:);
for j=2:hmax;
    suminvlikeerror = suminvlikeerror + 2*gaminvlike(j,:)*(1- (j-1)/hmax);
end;
suminvlikeerror = 100*sqrt(suminvlikeerror/sdraws)./meaninvlike' ;
toc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                          %
%                           Displays Main results                          %
%                                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin>6
    file=varargin{1};
else
    file=1;
end
if latex
    fprintf(file,'p log_like error\n');
    for i=1:size(p(:),1)
        fprintf(file,'%g\t%g\t%g \n',p(i),marg_data_dens(i),suminvlikeerror(i));
    end
    fprintf(file,'\\begin{table}[htbp]\n');
    fprintf(file,'\\caption{Posterior Results}\label{tab:}\n');
    fprintf(file,'$$\n');
    fprintf(file,'\\begin{tabular}{lccccc}\n');
    fprintf(file,'\\hline\n');
    fprintf(file,'\\hline\n');
    fprintf(file,'Param. & Mode & Mean & Median & Std. Dev. & 95\\%% Conf. Int. \\\\ \n');
    fprintf(file,'\\hline\n');
    if ~isempty(name)
    for i=1:ncoef;
        fprintf(file,'%12s & %6.2f & %6.2f & %6.2f & %6.2f & [%6.2f,%6.2f] \\\\ \n',name{i},theta_mode(i),theta_mean(i),theta_median(i),theta_std(i),theta_ci5(i,1),theta_ci5(i,2));
    end
    else
    for i=1:ncoef;
        fprintf(file,' & %6.2f & %6.2f & %6.2f & %6.2f & [%6.2f,%6.2f] \\\\ \n',theta_mode(i),theta_mean(i),theta_median(i),theta_std(i),theta_ci5(i,1),theta_ci5(i,2));
    end
    end
    fprintf(file,'\\hline\n');
    fprintf(file,'\\multicolumn{5}{c}{Average log marginal density:} & %12.6f \\\\\n',mean(marg_data_dens));
    fprintf(file,'\\hline\n');
    fprintf(file,'\\hline\n');
    fprintf(file,'\\end{tabular}\n');
    fprintf(file,'$$\n');
    fprintf(file,'\\end{table}\n');
    fprintf(file,'\n\n');
    for i=1:ncoef;
        fprintf(file,'%12s& %6.3f  \n',name{i},theta_mean(i));
        fprintf(file,'             & {\\scriptsize [%6.3f,%6.3f]}  \n',theta_ci5(i,1),theta_ci5(i,2));
    end

else
    fprintf(file,'-------------------------------------------------------------------------------------\n');
    fprintf(file,'       Param        Mode        Mean      Median       Stdev        5%%-L        5%%-H \n');
    fprintf(file,'-------------------------------------------------------------------------------------\n');
    for i=1:ncoef;
        fprintf(file,'%12s\t%6.2f\t%6.2f\t%6.2f\t%6.2f\t%6.2f\t%6.2f\n',name{i},theta_mode(i),theta_mean(i),theta_median(i),theta_std(i),theta_ci5(i,1),theta_ci5(i,2));
    end
    fprintf(file,'-------------------------------------------------------------------------------------\n');
    fprintf(file,'log(marginal data density):\n')
    for i=1:np
        fprintf(file,'    p=%4.2f                                                              %12.8f\n',p(i),marg_data_dens(i));
    end
    fprintf(file,'Mean log(marginal data density):                                        %12.8f\n',mean(marg_data_dens));
    fprintf(file,'-------------------------------------------------------------------------------------\n');
end