function [loglike,solution] = fdloglike(data,param,model)
solution= feval(model.name,param);
T       = size(data,1);

freqs   = 2*pi*(1:T)/T;
switch upper(model.filter)
    case 'HP'
        hplbd   = model.hp;    % HP filter parameter
        gain    = (4*hplbd*(1-cos(freqs)).^2)./(1+4*hplbd*(1-cos(freqs)).^2);
    case 'BP'
        gain    = zeros(T,1);
        gain(and(freqs>=model.wmin,freqs<=model.wmax))=1;
        gain((freqs>=(2*pi-model.wmax))&(freqs<=(2*pi-model.wmin)))= 1;        
    otherwise
        gain    = [0;ones(T-1,1)];
end
gain2   = gain.*gain;

if solution.id==0
    MX      = solution.mx;
    MY      = solution.my;
    ME      = solution.me;
    SIG     = solution.sig;
    im      = sqrt(-1);
    r2pi    = 1/(2*pi);
    OM      = ME*SIG*ME';
    loglike = 0;
    for t=1:T
        yzp     = data'*exp(im*freqs(t)*(1:T)');
        period  = yzp*yzp'/(2*pi*T);
        sp1     = (eye(length(MX))-MX*exp(-im*freqs(t)))\eye(length(MX));
        sp      = r2pi*(MY(model.select,:)*sp1*OM*sp1'*MY(model.select,:)');
        if gain2(t)>0
            sp      = gain2(t)*sp;
            detsp   = log(real(det(sp)));
            trsp    = real(trace(sp\period));
            loglike = loglike-(2*log(2*pi)+detsp+trsp)/2;
        end
    end
    if ~isreal(loglike)
        loglike=-1e20;
    end
else
    loglike=-1e20;
end