function [hess,ihess,sqrtihess,stdtheta,penalt]=hessian(theta_mode,data,fmodel,datapriors,bounds,fixed,itheta,ifixed);
%********************************************************
% Compute Hessian:
% element by element, fine tune with dxscale
%********************************************************

theta_mode  = theta_mode(:);
ncoef       = size(theta_mode,1);
ndx         = 6;                           % number of grid
dx          = exp(-(5:2:5+2*(ndx-1)))';    % grid
dxscale     = ones(ncoef,1);               % scale can be set differently across parameters
%
% set storage
%
hess        = zeros(ncoef,ncoef);
hessdiag    = zeros(ndx,1);
%
% Posterior at mode
%
nfix=size(fixed(:),1);

theta=zeros(ncoef+nfix,1);
theta(itheta)=theta_mode(:);
theta(ifixed)=fixed(:);
% jointprior  = fprior(theta,datapriors);
% [loglh,sol] = fdloglike(data,theta,fmodel);
% fx          = jointprior+loglh;
fx          = evalpost(theta,data,fmodel,datapriors,bounds);

%
% Compute diagonal elements first
%
for i=1:ncoef;
    for j=1:ndx;
        paradx      = theta_mode;
        parady      = theta_mode;
        paradx(i)   = paradx(i) + dx(j)*dxscale(i);      % forward
        parady(i)   = parady(i) - dx(j)*dxscale(i);      % backward
        theta(itheta)=paradx(:);
        theta(ifixed)=fixed(:);
%         jointprior  = fprior(theta,datapriors);
%         [loglh,sol] = fdloglike(data,theta,fmodel);
%         fdx         = jointprior+loglh;
        fdx          = evalpost(theta,data,fmodel,datapriors,bounds);

        
        theta(itheta)=parady(:);
%         jointprior  = fprior(theta,datapriors);
%         [loglh,sol] = fdloglike(data,theta,fmodel);
%         fdy         = jointprior+loglh;
        fdy          = evalpost(theta,data,fmodel,datapriors,bounds);
        hessdiag(j) = -( 2*fx - fdx - fdy)/(dx(j)*dxscale(i))^2;        
    end
    % negative of Hessian (average of values at 3rd and 4th grid) **/
    hess(i,i)    = -0.5*(hessdiag(3)+hessdiag(4));
end;
%
% Now compute the off-diagonal elements
% Make sure that correlations are between -1 and 1
% All invalid elements are reported in errorij

% errorij = [];
for i=1:ncoef;
    for j=i+1:ncoef;
        for k=1:ndx
            paradx      = theta_mode;
            parady      = theta_mode;
            paradx(i)   = paradx(i) + dx(k)*dxscale(i);
            parady(j)   = parady(j) - dx(k)*dxscale(j);
            paradxdy    = paradx;
            paradxdy(j) = paradxdy(j) - dx(k)*dxscale(j);
            theta(itheta)=paradx(:);
            theta(ifixed)=fixed(:);
%             jointprior  = fprior(theta,datapriors);
%             [loglh,sol] = fdloglike(data,theta,fmodel);
%             fdx         = jointprior+loglh;
           fdx          = evalpost(theta,data,fmodel,datapriors,bounds);

            theta(itheta)=parady(:);
%             jointprior  = fprior(theta,datapriors);
%             [loglh,sol] = fdloglike(data,theta,fmodel);
%             fdy         = jointprior+loglh;
            fdy          = evalpost(theta,data,fmodel,datapriors,bounds);
            theta(itheta)=paradxdy(:);
%             jointprior  = fprior(theta,datapriors);
%             [loglh,sol] = fdloglike(data,theta,fmodel);
%             fdxdy       = jointprior+loglh;
            fdxdy          = evalpost(theta,data,fmodel,datapriors,bounds);
            hessdiag(k) = -( fx - fdx - fdy + fdxdy )/(dx(k)*dx(k)*dxscale(i)*dxscale(j));
        end;
        %
        % negative of Hessian (average of values at 3rd and 4th grid)
        %
        hess(i,j) = -0.5*(hessdiag(3)+hessdiag(4));
        %
        % calculate correlation
        %
        if or(hess(i,i)==0,hess(j,j)==0);    % zero variance
            corrij  = 0;
        else                                              % (covariance)/(standard deviations)
            corrij  = hess(i,j)/sqrt(hess(i,i)*hess(j,j));
        end;
        %
        % adjust correlation
        %
        if abs(corrij)>0.975;                                % close to 1
            hess(i,j)    = 0.975*sign(hess(i,j))*sqrt(hess(i,i)*hess(j,j));  % 
            %errorij      = [errorij; [i j corrij]];
        elseif abs(corrij)<0.005;                           % close to 0
            hess(i,j)    = 0;
        end;
        %
        % symmetrize
        %
        hess(j,i) = hess(i,j);
    end
end
%********************************************************
% Compute covariance matrix and penalty (posterior mode)
%********************************************************
%
% Compute the inverse of H using Singular Value Decomposition (SVD) 
%
[u,s,v]     = svd(hess);
is          = diag(1./diag(s));
ihessdet    = prod(diag(is));
ihess       = v*is*u';          % inverse of negative Hessian: covariance
ihess       = (ihess+ihess')/2; % Guarantees symmetry
sqrtihess   = v*sqrt(is);       % square root of covariance matrix at posterior mode
stdtheta    = sqrt(diag(ihess));% std of estimates at posterior mode
penalt      = 0.5*ncoef*log(2*pi) + 0.5*log(ihessdet); % posterior mode penalty 
