function smooth=kalman_smoother(data,model,varargin)
%
% Evaluate the Kalman Smoother for the state space model
%
% Y(t)  = PI X(t-1) + w(t)
% X(t)  = MX X(t-1) + ME e(t)
%
% with:
%
% E(e(t)e'(s))=SE for t=s, 0 otherwise
% E(w(t)w'(s))=R  for t=s, 0 otherwise
%
% and data are collected for Y.
%
MY          = model.my;
ny          = size(MY,1);
MX          = model.mx;
nx          = size(MX,2);
ME          = model.me;
ne          = size(ME,2);
SE          = model.sig;
% MY          = [MY zeros(ny,ne)];
% MX          = blkdiag(MX,zeros(ne,ne));
% ME          = [ME;eye(ne)];
if(isfield(model,'sigy'))
    R       = model.sigy;
else
	R       = 0;
end
[long,nvar] = size(data);
nstate      = size(MX,1);
%
% kalman filter
%
Q       = ME*SE*ME';                    % Covariance matrix of error term
xtt     = zeros(nstate,long);           % Initial values
yttm1   = zeros(ny,long);               % Initial values
xttm1   = zeros(nstate,long+1);         % Initial values
Ptt     = zeros(nstate*nstate,long);    % Initial values
Pttm1   = zeros(nstate*nstate,long+1);  % Initial values
Wy      = zeros(ny*ny,long);            % Initial values
if max(abs(eig(MX)))<=1;
    if nargin>2
        xttm1(:,1)  = varargin{1};
    else
        xttm1(:,1)  = 0;
    end
    Vttm1       = reshape((eye(nstate*nstate)-kron(MX,MX))\Q(:),nstate,nstate);
    Pttm1(:,1)  = Vttm1(:);
    for t=1:long;   % See Hamilton pp. 377-381
        tmp         = xttm1(:,t)+Vttm1*MY'*((MY*Vttm1*MY'+R)\(data(t,:)'-MY*xttm1(:,t)));
        xtt(:,t)    = tmp;
        xttm1(:,t+1)= MX*xtt(:,t);
        yttm1(:,t)  = MY*xttm1(:,t);
        Vtt         = Vttm1-Vttm1*MY'*((MY*Vttm1*MY'+R)\(MY*Vttm1));
        Ptt(:,t)    = Vtt(:);
        Vttm1       = MX*Vtt*MX'+Q;
        Pttm1(:,t+1)= Vttm1(:);
        tmp         = MY*Vttm1*MY';
        Wy(:,t)     = tmp(:);
    end

    
    xtT             = zeros(nstate,long);
    PtT             = zeros(nstate*nstate,long);
    PtT(:,long)     = Ptt(:,long);
    xtT(:,long)     = xtt(:,long);
    for t=long-1:-1:1;
        tmp     = reshape(Ptt(:,t),nstate,nstate);
        tmp1    = reshape(PtT(:,t+1),nstate,nstate);
        tmpm1   = reshape(Pttm1(:,t+1),nstate,nstate)+1e-8*eye(nstate);
        J       = tmp*MX'*(tmpm1\eye(nstate));
        xtT(:,t)= xtt(:,t)+J*(xtT(:,t+1)-xttm1(:,t+1));
        V       = tmp+J*(tmp1-tmpm1)*J';
        PtT(:,t)= V(:);
    end
    smooth.x    = xtT(1:nx,:)';   
    tmpe        = (xtT(:,2:long)-MX*xtT(:,1:long-1))';
    smooth.e    = [zeros(1,ne);tmpe*ME];
    smooth.p    = PtT';
    smooth.yf   = yttm1';
    smooth.vyf  = Wy';
else
    smooth=[];
end
