function [loglh,sol]=loglike(data,param,model)
%
% Evaluate the log-likelihood of a state space model of the form
%
% Y(t)  = PI X(t-1)
% X(t)  = MX X(t-1) + ME e(t)
%
% with:
%
% E(e(t)e'(s))=SE for t=s, 0 otherwise
%
llm         =-1e20;
sol         = feval(model.name,param);
if sol.id==0
    id	= and(all(~isinf(sol.mx)),all(~isnan(sol.mx)));
else
    id  = 0;
end

if id
    MY          = sol.my(model.select,:);
    MX          = sol.mx;
    ME          = sol.me;
    SE          = sol.sig;
    [T,nvar]    = size(data);
    nstate      = size(sol.mx,1);
    % kalman filter
    %----------------
    Q   = ME*SE*ME';    % covariance matrix of error term
    Xhat= zeros(nstate,1);      %initial values

    if max(abs(eig(MX)))<=1;
        Phat=disclyap_fast(MX,Q);
%         Phat      = Q;%reshape((eye(nstate*nstate)-kron(MX,MX))\Q(:),nstate,nstate);
        loglh   = 0;
        for t=1:T
            [Xhat,Phat,llh] = kfilter(data(t,:)',MY,0,MX,Xhat,Phat,0,Q);
            loglh           = loglh+real(llh);
        end
    else
        loglh=llm;
    end
else
    loglh=llm;
end
