function logIG1pdf = lpdfig(x,mu,sig)

% computing the deg and scale that parameterize the IG1 density as a
% function of mean and std
[scale,deg]=inverse_gamma_specification(mu,sig,1); 

% evaluating the density
if x>0
[junk logIG1pdf]=pdf_igone(x,deg,scale);
else 
    logIG1pdf=NaN;
end
return


function [s,nu] = inverse_gamma_specification(mu,sigma,type)
%
% mu    --> expectation
% sigma --> standard deviation
% type = 1 --> Inverse Gamma 1
% type = 2 --> Inverse Gamma 2
%
% stephane.adjemian@cepremap.cnrs.fr [01/14/2004]


sigma2 = sigma^2;
mu2 = mu^2;

if type == 2;       % Inverse Gamma 2   
   nu   = 2*(2+mu2/sigma2);
   s    = 2*mu*(1+mu2/sigma2);
elseif type == 1;   % Inverse Gamma 1 
    if sigma2 < Inf;
      nu = sqrt(2*(2+mu2/sigma2));
      nu2 = 2*nu;
      nu1 = 2;
      err = 2*mu2*gamma(nu/2)^2-(sigma2+mu2)*(nu-2)*gamma((nu-1)/2)^2;
      while abs(nu2-nu1) > 1e-12
	if err > 0
	  nu1 = nu;
	  if nu < nu2
	    nu = nu2;
	  else
	    nu = 2*nu;
	    nu2 = nu;
	  end
	else
	  nu2 = nu;
	end
	nu =  (nu1+nu2)/2;
	err = 2*mu2*gamma(nu/2)^2-(sigma2+mu2)*(nu-2)*gamma((nu-1)/2)^2;
      end
      s = (sigma2+mu2)*(nu-2);
    else;
        nu  = 2;
        s   = 2*mu2/pi;
    end;   
else;
   s  = -1;
   nu = -1;
end;

% 01/18/2004 MJ replaced fsolve with secant
%               suppressed chck
%	        changed order of output parameters
return

  function [pd,logpdf]=pdf_igone(sd,v,s);
% function [pdf,logpdf]=pdf_igone(sd,v,s);
% Computes the pdf of an InverseGamma1 
% with degrees of freedom v and scale parameter s 
% This is for the standard deviation 
% Use invwishlub.m for the pdf of an IG2 
% AJ 4/29/04
num=log(2) - (v+1)*log(sd) -s/( 2*(sd^2) ); 
den=gammaln(v/2) + (v/2)*log(2/s); 
logpdf=num-den; 
pd=exp(logpdf); 
return