function [theta,p_theta]=metro_hast(theta_mode,invhess,data,datapriors,fmodel,metro,bounds,fixed,itheta,ifixed,varargin)

NDraws  = metro.ndraws;
discard = metro.discard;
step    = metro.step;
c0      = metro.c0;
c       = metro.c;
sdraws  = NDraws+discard;
kept    = discard+1:step:sdraws;
ncoef   = size(theta_mode(:),1);
nfix    = size(fixed(:),1);
if nargin<7;
    bounds  = repmat([-1e20 1e20],size(theta_mode(:),1),1);
end

%------------------------------
% METROPOLIS-HASTING ALGORITHM
%------------------------------
param   = zeros(ncoef+nfix,1);
param(ifixed)= fixed(:);
theta   = zeros(sdraws,ncoef);
p_theta = zeros(sdraws,1);
% e       = mvnrnd(zeros(1,ncoef),invhess*c*c,sdraws);
% p_rej   = unifrnd(0,1,sdraws,1);
%
% Initial value for the algorithm
%
if nargin>10
    rng(varargin{1})
end
crit    = -1e20;
while ~(crit>-1e20)
    e0        	= mvnrnd(zeros(1,ncoef),invhess*c0*c0);
    theta(1,:)  = theta_mode(:)'+e0;
    %
    % calculate p(theta) for initial value
    %    
    param(itheta)= theta(1,:)';
    p_theta(1)  = evalpost(param,data,fmodel,datapriors,bounds);
    crit        = p_theta(1);
end
accept  = 0;
tic
k=2;

for i=2:sdraws;
    % draw z from q(.|theta_t)
    % q is a multivariate normal with mean zero and inverse hessian as covariance matrix
    valid=1;
    while valid
        e               = mvnrnd(zeros(1,ncoef),invhess*c*c);
        theta_st        = theta(i-1,:)+e;
        p_rej           = rand(1);
        % calculate p(theta*)
        param(itheta)   = theta_st(:);
        [p_theta_st,sol]= evalpost(param,data,fmodel,datapriors,bounds);
        if and((p_theta_st>-1e20),(sol.id==0))
            % calculate acceptance probability
            r   = min(exp(p_theta_st-p_theta(i-1)),1);
            %
            % accept/reject draws
            %
            if (p_rej<= r); % accept because of draws
                theta(i,:)  = theta_st;
                p_theta(i)  = p_theta_st;
                accept      = accept+1;
            else                        % reject because of draws
                theta(i,:)  = theta(i-1,:);
                p_theta(i)  = p_theta(i-1);
            end;
            valid=0;
        end;
    end
    if k==2500;
        fprintf('Simulation: %7d  Accept: %6.2f%%\n',i,100*accept/i);
        k   = 1;
    else
        k   = k+1;
    end
end;
theta       = theta(kept,:);
p_theta     = p_theta(kept);
fprintf('Acceptance rate: %6.2f%%\n',100*(accept/sdraws));