function sol=solve_beliefs(model,varargin)
if nargin>1
    nforecasts=varargin{1};
else
    nforecasts=5;
end

% Measurement equation

Myy = model.myy;
Myx = model.myx;
MyX = model.myX;
MyY = model.myY;
Myf = model.myf;
MyF = model.myF;
Mys = model.mys;

% Backward looking part

Mxx0= model.mxx0;
Mxx1= model.mxx1;
MxX1= model.mxX1;
Mxy1= model.mxy1;
MxY1= model.mxY1;
Mxf1= model.mxf1;
MxF1= model.mxF1;
Mxs1= model.mxs1;
nb  = size(model.mxx0,1);
% Forward looking part

Mff0= model.mff0;
MfF0= model.mfF0;
Mff1= model.mff1;
MfF1= model.mfF1;
Mfx0= model.mfx0;
Mfx1= model.mfx1;
MfX1= model.mfX1;
Mfy0= model.mfy0;
MfY0= model.mfY0;
Mfy1= model.mfy1;
MfY1= model.mfY1;
Mfs0= model.mfs0;
Mfs1= model.mfs1;
nf  = size(model.mff0,1);
% Shocks
R   = model.r;
ns  = size(R,1);
Q   = model.q;
nx  = size(Q,1);
DELT= model.Delta;

% Trsnformation of matrices

Mx  = Myy\Myx;
MX  = Myy\(Myx+MyX);
MEY = Myy\MyY;
Mf  = Myy\Myf;
MF  = Myy\(Myf+MyF);
Ms  = Myy\Mys;
I   = eye(size(MEY,1));

Nx  = Mxx0\Mxx1;
NX  = Mxx0\(Mxx1+MxX1);
Ny  = Mxx0\Mxy1;
NY  = Mxx0\(Mxy1+MxY1);
Nf  = Mxx0\Mxf1;
NF  = Mxx0\(Mxf1+MxF1);
Ns  = Mxx0\Mxs1;

Pf0 = Mff0;
Pf1 = Mff1+Mfx0*Nf;
PF0 = MfF0;
PF1 = Mff1+MfF1+Mfx0*NF;
Px  = Mfx1+Mfx0*Nx;
PX  = Mfx1+MfX1+Mfx0*NX;
Py0 = Mfy0;
PY0 = Mfy0+MfY0;
Py1 = Mfy1+Mfx0*Ny;
PY1 = Mfy1+MfY1+Mfx0*NY;
Ps  = Mfs0*R+Mfs1+Mfx0*Ns;

% STEP 1: Solve the representative agent model

M0  = [ eye(nb) zeros(nb,nf);
    -PY0*((I-MEY)\MX)  Pf0-PF0-PY0*((I-MEY)\MF)];

M1  = [ NX+NY*((I-MEY)\MX) NF+NY*((I-MEY)\MF)
    PX+PY1*((I-MEY)\MX) PF1+PY1*((I-MEY)\MF)];

M2  = [Ns+NY*((I-MEY)\Ms);Ps+PY1*((I-MEY)\Ms)+PY0*((I-MEY)\Ms)*R];

[s,t,q,z]   = qz(M0,M1);            % Generalized Eigenvalues
[s,t,q,z]   = ordqz(s,t,q,z,'udo'); % Reorders eigenvalues

z21     = z(nb+1:nb+nf,1:nb);
z11     = z(1:nb,1:nb);

if rank(z11)<nb;
    warning('Invertibility condition violated')
    sol.id  = -1;
else
    z11i    = z11\eye(nb);
    s11     = s(1:nb,1:nb);
    t11     = t(1:nb,1:nb);
    
    if or(abs(t(nb,nb))>abs(s(nb,nb)),abs(t(nb+1,nb+1))<abs(s(nb+1,nb+1)))
        warning('Wrong number of stable eigenvalues.');
        sol.id=1;
    else
        
        dyn     = s11\t11;
        GFX      = z21*z11i;
        if max(max(imag(GFX)))<1e-6;
            GFX = real(GFX);
        end
        MUX    = z11*dyn*z11i;
        if max(max(imag(MUX)))<1e-6;
            MUX  = real(MUX);
        end
        M       = [eye(nb);GFX];
        B       = 1:nb;
        F       = nb+1:nb+nf;
        
        PSI0    =-(M0(B,B)+M0(B,F)*GFX)\M0(B,F);
        PSI1    = (M0(B,B)+M0(B,F)*GFX)\M1(B,F);
        PSI2    = (M0(B,B)+M0(B,F)*GFX)\M2(B,:);
        G0      = (M0(F,B)+M0(F,F)*GFX)*PSI0+M0(F,F);
        G1      = (M0(F,B)+M0(F,F)*GFX)*PSI1-M1(F,F);
        G2      = M2(F,:)-(M0(F,B)+M0(F,F)*GFX)*PSI2;
        Qtmp    = (kron(R',G0)+kron(eye(ns),G1));

        tmp     = Qtmp\G2(:);
        GFS     = reshape(tmp,nf,ns);
        MUS     = PSI0*GFS*R+PSI1*GFS+PSI2;
        LBX     = (I-MEY)\(MX+MF*GFX);
        LBS     = (I-MEY)\(Ms+MF*GFS);
        sol.id  = 0;
        % Step 2: Solve the beliefs part
        % A GFxi Q +B GFxi = C
        A   = (Pf0-PF0)-PY0*((I-MEY)\MF);
        B   = ((Pf0-PF0)*GFX*NY-PY1-PY0*LBX*NY)*((I-MEY)\MF)+(Pf0-PF0)*GFX*NF-PF1-PY0*LBX*NF;
        C0  = ((PY0-Py0)*LBS+PY0*((I-MEY)\((MF-Mf)*GFS+MEY*LBS)))*DELT*Q;
        C1  = ((Pf0-PF0)*GFX*NY-PY1-PY0*LBX*NY)*((I-MEY)\((MF-Mf)*GFS+MEY*LBS))*DELT;
        C   = C0-C1;
        tmp = kron(Q',A)+kron(eye(nx),B);
        

        vGxi= tmp\C(:);
        GFxi= reshape(vGxi,nf,nx);
        LBxi= (I-MEY)\(MF*GFxi+(MF-Mf)*GFS*DELT+MEY*LBS*DELT);
        MUxi= NF*GFxi+NY*LBxi;
        
        sol.mx  = [MUX MUS MUxi;zeros(ns,nb) R zeros(ns,nx);zeros(nx,nb+ns) Q];
        sol.me  = [zeros(nb,ns+nx);eye(ns+nx)];
        sol.my  = [LBX LBS LBxi;GFX GFS GFxi];
        sol.mxx = MUX;
        sol.mxs = MUS;
        sol.mxxi= MUxi;
        sol.myx = [LBX;GFX];
        sol.mys = [LBS;GFS];
        sol.myxi= [LBxi;GFxi];
        sol.r   = R;
        sol.q   = Q;
        sol.delt= DELT;
        sol.err = [LBS;GFS]*DELT;
        %% To get forecasts
        nx  = size(sol.mxx,1);
        ny  = size(sol.my,1);
        ns  = size(sol.r,1);
        nxi = size(sol.q,1);
        
        MZ  = [sol.mxx sol.mxs sol.mxs*sol.delt+sol.mxxi;zeros(ns,nx) sol.r zeros(ns,nxi);zeros(nxi,nx+ns) sol.q];
        MXi = eye(nx+ns+nxi);
        Ri  = eye(ns);
        Qi  = eye(nxi);
        RQi = blkdiag(R,Q);
        S   = eye(ns+nxi);
        for i=1:nforecasts
            MYx = sol.myx*MXi(1:nx,:);
            MYz = sol.mys*Ri;
            MYxi= (sol.mys*sol.delt+sol.myxi)*Qi;
            sol.myf{i}  = MYx+[zeros(ny,nx) MYz MYxi];
            MXi = MZ*MXi;
            Ri  = sol.r*Ri;
            Qi  = sol.q*Qi;
            sol.mxf{i}  = blkdiag(zeros(nx,nx),blkdiag(eye(ns),DELT)*S);
            S   = eye(ns+nxi)+RQi*S;
        end
        sol.mz  = MZ;        
    end
end

