function sol=solve_inertia(model,varargin)
if nargin>1
    verb_   = varargin{1};
else
    verb_   = 0;
end
exoxi_  = model.exobelief;
% *Measurement equation*
Myy = model.myy;
Myx = model.myx;
MyX = model.myX;
MyY = model.myY;
Myf = model.myf;
MyF = model.myF;
Mys = model.mys;
ny  = size(Myy,1);
% *Backward looking part*
Mxx0= model.mxx0;
Mxx1= model.mxx1;
MxX1= model.mxX1;
Mxy1= model.mxy1;
MxY1= model.mxY1;
Mxf1= model.mxf1;
MxF1= model.mxF1;
Mxs1= model.mxs1;
nb  = size(model.mxx0,1);
% *Forward Looking Part*
Mff0= model.mff0;
MfF0= model.mfF0;
Mff1= model.mff1;
MfF1= model.mfF1;
Mfx0= model.mfx0;
Mfx1= model.mfx1;
MfX1= model.mfX1;
Mfy0= model.mfy0;
MfY0= model.mfY0;
Mfy1= model.mfy1;
MfY1= model.mfY1;
Mfs0= model.mfs0;
Mfs1= model.mfs1;
nf  = size(model.mff0,1);
% *Shocks*
R   = model.r;
ns  = size(R,1);
A   = model.a;
B   = model.b;
C   = model.c;
Me  = model.me;
ne  = size(Me,2);
nxi = size(A,1);
DELT= model.Delta;

% *Rewriting the Model*
Mx  = Myy\Myx;
MX  = Myy\(Myx+MyX);
MEY = Myy\MyY;
Mf  = Myy\Myf;
MF  = Myy\(Myf+MyF);
Ms  = Myy\Mys;
I   = eye(size(MEY,1));

Nx  = Mxx0\Mxx1;
NX  = Mxx0\(Mxx1+MxX1);
Ny  = Mxx0\Mxy1;
NY  = Mxx0\(Mxy1+MxY1);
Nf  = Mxx0\Mxf1;
NF  = Mxx0\(Mxf1+MxF1);
Ns  = Mxx0\Mxs1;

Pf0 = Mff0;
Pf1 = Mff1+Mfx0*Nf;
PF0 = MfF0;
PF1 = Mff1+MfF1+Mfx0*NF;
Px  = Mfx1+Mfx0*Nx;
PX  = Mfx1+MfX1+Mfx0*NX;
Py0 = Mfy0;
PY0 = Mfy0+MfY0;
Py1 = Mfy1+Mfx0*Ny;
PY1 = Mfy1+MfY1+Mfx0*NY;
Ps  = Mfs0*R+Mfs1+Mfx0*Ns;

D   = B-C*R;
F   = DELT*A;
G   = DELT*D;
H   = eye(ns)+DELT*C;
J   = DELT;
%%
%% STEP 1: Solve the deterministic part of the representative agent model

M0  = [ eye(nb) zeros(nb,nf);
    -PY0*((I-MEY)\MX)  Pf0-PF0-PY0*((I-MEY)\MF)];

M1  = [ NX+NY*((I-MEY)\MX) NF+NY*((I-MEY)\MF)
    PX+PY1*((I-MEY)\MX) PF1+PY1*((I-MEY)\MF)];

[s,t,q,z]   = qz(M0,M1);            % Generalized Eigenvalues
[s,t,~,z]   = ordqz(s,t,q,z,'udo'); % Reorders eigenvalues

z21     = z(nb+1:nb+nf,1:nb);
z11     = z(1:nb,1:nb);

if rank(z11)<nb
    warning('Representative Agent problem: Invertibility condition violated')
    sol.id  = -1;
else
    z11i    = z11\eye(nb);
    s11     = s(1:nb,1:nb);
    t11     = t(1:nb,1:nb);
    
    if or(abs(t(nb,nb))>abs(s(nb,nb)),abs(t(nb+1,nb+1))<abs(s(nb+1,nb+1)))
        warning('Representative Agent problem: Wrong number of stable eigenvalues.');
        if verb_
            for i=1:nb+nf
                lb=t(i,i)/s(i,i);
                if ~isreal(lb)
                    fprintf('%8.6f%+8.6fi\t%8.6f\n',real(lb),imag(lb),abs(lb))
                else
                    fprintf('%8.6f%\t%8.6f\n',lb,abs(lb))
                end
            end
        end
        sol.id=1;
    else
        
        GAMX    = z21*z11i;
        if max(max(imag(GAMX)))<1e-6
            GAMX    = real(GAMX);
        end
        dyn     = s11\t11;
        OMX    = z11*dyn*z11i;
        if max(max(imag(OMX)))<1e-6
            OMX     = real(OMX);
        end
        LBX     = (I-MEY)\(MX+MF*GAMX);
        sol.id  = 0;
    end
end
%% STEP 2: Solve the deterministic part of the individual agent
if sol.id==0
    M0  = [ eye(nb) zeros(nb,nf);
        -Py0*Mx  Pf0-Py0*Mf];
    
    M1  = [ Nx+Ny*Mx Nf+Ny*Mf
        Px+Py1*Mx Pf1+Py1*Mf];
    
    [s,t,q,z]   = qz(M0,M1);            % Generalized Eigenvalues
    [s,t,~,z]   = ordqz(s,t,q,z,'udo'); % Reorders eigenvalues
    
    z21     = z(nb+1:nb+nf,1:nb);
    z11     = z(1:nb,1:nb);
    
    if rank(z11)<nb
        warning('Individual''s Problem: Invertibility condition violated')
        sol.id  = -1;
    else
        z11i    = z11\eye(nb);
        s11     = s(1:nb,1:nb);
        t11     = t(1:nb,1:nb);
        
        if or(abs(t(nb,nb))>abs(s(nb,nb)),abs(t(nb+1,nb+1))<abs(s(nb+1,nb+1)))
            warning('Individual''s Problem:Wrong number of stable eigenvalues.');
            sol.id=1;
        else
            
            dyn     = s11\t11;
            GAMx    = z21*z11i;
            if max(max(imag(GAMx)))<1e-6
                GAMx    = real(GAMx);
            end
            OMx    = z11*dyn*z11i;
            if max(max(imag(OMx)))<1e-6
                OMx     = real(OMx);
            end
            LBx     = Mx+Mf*GAMx;
            sol.id  = 0;
        end
    end
end

%% Step 3: Solves the stochastic part
if sol.id==0
    TMP     = ((PF0-Pf0)*GAMX+PY0*LBX);
    % a) Obtains GAMz=PSIzz LBz
    tmp0    = Pf1+Py0*LBx*Nf-Pf0*GAMx*Nf;
    tmp1    = Pf0*GAMx*Ny-Py0*LBx*Ny-Py1;
    PSIzz   = tmp0\tmp1;
    % b) Obtains GAMzb=PSIzbz LBz
    tmp0    = (TMP)*NF+PF1;
    % tmp1    = Py0*(LBX-LBx)*(Ny+Nf*PSIzz)-PY1-TMP*(NY+NF*PSIzz)-PF1*PSIzz;
    tmp1    = TMP*(NY+NF*PSIzz)+PF1*PSIzz+PY1;
    PSIzbz  = -tmp0\tmp1;
    % c) LBxi   = PSIxiz*LBz*F+PSIxixi GAMxi
    I       = eye(size(MEY,1));
    PSIxiz  = (I-MEY)\(MEY+MF*(PSIzz+PSIzbz)-Mf*PSIzz);
    PSIxixi = (I-MEY)\MF;
    % d) LBs   = PSIsz*LBz*G+PSIse*GAMe*R+PSIss*GAMs
    PSIsz   = (I-MEY)\(MEY+MF*(PSIzz+PSIzbz)-Mf*PSIzz);
    PSIse   =-(I-MEY)\MF;
    PSIss   = (I-MEY)\MF;
    % e) Vec(GAMxi) = THxilb Vec(LBz)
    I       = eye(nxi);
    Axixi   = TMP*(NY*PSIxixi+NF)+PF1+PY1*PSIxixi;
    Bxixi   = PF0-Pf0+PY0*PSIxixi;
    B1xilb  = TMP*NY*PSIxiz+PY1*PSIxiz;
    B2xilb  = (PF0-Pf0)*PSIzbz+PF0*PSIzz+PY0*PSIxiz+PY0-Py0;
    tmp0    = kron(I,Axixi)+kron(A',Bxixi);
    tmp1    = kron(F',B1xilb)+kron((F*A)',B2xilb);
    THxiz   = -tmp0\tmp1;
    % f) LBz equation
    Alblb   = Mf*PSIzz-eye(ny);
    Blblb   = MF*(PSIzz+PSIzbz)-Mf*PSIzz+MEY;
    Albe    = MF;
    Flblb   = kron(H',Blblb)+kron(eye(ns),Alblb);
    Flbe    = kron(eye(ne),Albe);
    Dlb     = Ms(:);
    % g) GAMs equation
    Ass     = TMP*(NY*PSIss+NF)+PF1+PY1*PSIss;
    Bss     = PF0-Pf0+PY0*PSIss;
    B1se    = (TMP*NY+PY1)*PSIse;
    B2se    = PY0*PSIse;
    Bsxi    = PF0-Pf0+PY0*PSIxixi;
    W0      = (TMP*NY+PY1)*PSIsz;
    W1      = PF0*PSIzz-Py0;
    W2      = (PF0-Pf0)*PSIzz;
    W3      = (PF0-Pf0)*PSIzbz+PY0;
    W4      = PY0*PSIsz;
    W5      = PY0*PSIxiz;
    Fss     = kron(eye(ns),Ass)+kron(R',Bss);
    Fse     = kron(R',B1se)+kron((R*R)',B2se);
    Fslb    = kron(B',Bsxi)*THxiz+kron(G',W0)+kron(((G+H*R-R)*R+F*B)',W1)+kron((R*R)',W2)+kron(((G+H*R)*R+F*B)',W3)+kron((G*R)',W4)+kron((F*B)',W5);
    tmp     = TMP*Ns*R+Ps*R;
    Ds      = tmp(:);
    % h) GAMe equation
    Aee     = TMP*NF+PF1;
    Bee     = PY0*PSIse;
    Aes     = PF0-Pf0+PY0*PSIss;
    Bexi    = PF0-Pf0+PY0*PSIxixi;
    W0      = (PF0-Pf0)*PSIzbz+PF0*PSIzz+PY0-Py0;
    W1      = (PF0-Pf0)*PSIzz-PF0*PSIzz+Py0;
    W2      = PY0*PSIxiz;
    W3      = PY0*PSIsz;
    Fee     = kron(eye(ne),Aee)+kron(R',Bee);
    Fes     = kron(eye(ns),Aes);
    Felb    = kron(C',Bexi)*THxiz+kron((G+H*R+F*C)',W0)+kron(R',W1)+kron((F*C)',W2)+kron(G',W3);
    tmp     = TMP*Ns+Ps;
    De      = tmp(:);
    % Obtains final solution
    n1      = size(Flblb,1);
    n2      = size(Fss,2);
    TMP0    = [Flblb zeros(n1,n2) Flbe;Fslb Fss Fse;Felb Fes Fee];
    DD      = eig(TMP0);
    %     if sum(real(DD)<0)~=nf*ns
    %         sum(real(DD)<0)
    %         sol.id=1;
    %         sol.mx=[];
    %         sol.my=[];
    %         sol.me=[];
    %         sol.sig=[];
    %         return
    %     end
    
    TMP1    =-[Dlb;Ds;De];
    VEC     = TMP0\TMP1;
    LBz     = reshape(VEC(1:ny*ns),ny,ns);
    GAMz    = PSIzz*LBz;
    GAMzb   = PSIzbz*LBz;
    GAMs    = reshape(VEC(ny*ns+(1:nf*ns)),nf,ns);
    GAMe    = reshape(VEC(ny*ns+nf*ns+(1:nf*ne)),nf,ne);
    GAMxi   = reshape(THxiz*LBz(:),nf,nxi);
    LBs     = PSIsz*LBz*G+PSIse*GAMe*R+PSIss*GAMs;
    LBxi    = PSIxiz*LBz*F+PSIxixi*GAMxi;
    
    % OMX     = NX+NY*LBX+NF*GAMX;
    % OMx     = Nx+Ny*LBx+Nf*GAMx;
    OMs     = Ns*R+NF*GAMs+NY*LBs;
    OMe     = NF*GAMe+Ns;
    OMz     = Ny*LBz+Nf*GAMz;
    OMzb    = (NY-Ny)*LBz+(NF-Nf)*GAMz+NF*GAMzb;
    OMxi    = NY*LBxi+NF*GAMxi;
    PHIx    = GAMx*OMx;
    PHIX    = GAMX*OMX;
    PHIz    = GAMx*OMz;
    PHIzb   = (GAMX-GAMx)*OMz+GAMX*OMzb;
    PHIs    = GAMX*OMs+(GAMz*R+GAMzb*(G+H*R)+GAMs)*R+(GAMzb*F+GAMxi)*B;
    PHIxi   = GAMX*OMxi+(GAMzb*F+GAMxi)*A;
    PHIe    = GAMX*OMe+(GAMz*R+GAMzb*(G+H*R)+GAMs)+(GAMzb*F+GAMxi)*C;
    switch exoxi_
        case 1
            I       = eye(ny);
            % d) LBet = PSIetz*LBz*J+PSIetet GAMet
            PSIetz  = (I-MEY)\(MEY+MF*(PSIzz+PSIzbz)-Mf*PSIzz);
            PSIetet = (I-MEY)\MF;
            % f) Vec(GAMet) = THetlb Vec(LBz)
            Aetet   = TMP*(NY*PSIetet+NF)+PF1+PY1*PSIetet;
            Betet   = PF0-Pf0+PY0*PSIxixi;
            B1etlb  = (TMP*NY+PY1)*PSIetz;
            B2etlb  = (PF0-Pf0)*PSIzbz+PF0*PSIzz+PY0-Py0+PY0*PSIxiz;
            
            GAMet   = -Aetet\(Betet*GAMxi+B1etlb*LBz*J+B2etlb*LBz*F);
            LBet    = PSIetz*LBz*J+PSIetet*GAMet;
            OMet    = NY*LBet+NF*GAMet;
            PHIet   = GAMX*OMet+(GAMzb*F+GAMxi);
            
            sol.my  = [ LBX LBs-LBet*(B-C*R) LBxi-LBet*A LBz-LBet*C LBet;
                GAMX GAMs-GAMe*R-GAMet*(B-C*R) GAMxi-GAMet*A GAMz+GAMzb+GAMe-GAMet*C GAMet];
            sol.mx  = [ OMX OMs-OMe*R-OMet*(B-C*R) OMxi-OMet*A OMz+OMzb+OMe-OMet*C OMet;
                zeros(ns+nxi,nb+ns+nxi) eye(ns+nxi);
                zeros(ns+nxi,nb+ns+nxi) [R zeros(ns,nxi);B A]];           
            
            sol.me  = [ zeros(nb+ns+nxi,ne+nxi);Me zeros(ns,nxi);C eye(nxi)];
            sol.err = [zeros(ny+nf,nb+2*ns+nxi) [-LBz*DELT;-(GAMz+GAMzb)*DELT]];
            sol.emy = sol.my-sol.err;
            sol.erri= [zeros(ny+nf,nb+2*ns+nxi) [zeros(ny,nxi);-GAMzb*DELT]];
            sol.emyi= sol.my-sol.erri;
        otherwise
            sol.my  = [ LBX LBs LBxi LBz zeros(ny,nxi);
                GAMX GAMs-GAMe*R GAMxi GAMz+GAMzb+GAMe  zeros(nf,nxi)];
            sol.mx  = [ OMX OMs-OMe*R OMxi OMz+OMzb+OMe zeros(nb,nxi);
                zeros(ns+nxi,nb+ns+nxi) eye(ns+nxi);
                zeros(ns+nxi,nb+ns+nxi) [R zeros(ns,nxi);B A]];
            sol.me  = [zeros(nb+ns+nxi,ne);Me;C];
            sol.err = [zeros(ny+nf,nb+2*ns+nxi) [-LBz*DELT;-(GAMz+GAMzb)*DELT]];
            sol.erri= [zeros(ny+nf,nb+2*ns+nxi) [zeros(ny,nxi);-GAMzb*DELT]];
            sol.emy = sol.my-sol.err;
            sol.emyi= sol.my-sol.erri;
            sol.mxx = [ OMX (OMz+OMzb)*R+OMs OMxi;
                zeros(ns,nb) R zeros(ns,nxi)
                zeros(nxi,nb) B A];
            sol.mxe = [ OMz+OMzb+OMe;Me;C];
            sol.myx = [ LBX LBz*R+LBs LBxi;
                GAMX (GAMz+GAMzb)*R+GAMs GAMxi];
            sol.mye = [LBz;GAMz+GAMzb+GAMe];
    end
else
    sol.id=1;
    sol.mx=[];
    sol.my=[];
    sol.me=[];
    sol.sig=[];
end
%% To verify that all equations are indeed satisfied.
switch verb_
    case 1
        resx    = Pf0*PHIx-Px-Pf1*GAMx-Py0*LBx*OMx-Py1*LBx;
        resX    = (PF0-Pf0)*PHIX+PF1*GAMX+PX+PY0*LBX*OMX+PY1*LBX;
        resz    = Pf0*PHIz-Pf1*GAMz-Py0*LBx*OMz-Py1*LBz;
        reszb   = (PF0-Pf0)*PHIzb+PF0*PHIz+(PF1-Pf1)*GAMz+PF1*GAMzb+(PY0-Py0)*LBX*OMz+PY0*LBX*OMzb+(PY1-Py1)*LBz;
        resxi   = (PF0-Pf0)*PHIxi+PF0*GAMz*F*A+PF1*GAMxi+PY0*(LBX*OMxi+LBxi*A)+(PY0-Py0)*LBz*F*A+PY1*LBxi;
        reslb   = LBz-Ms-MF*(GAMz+GAMzb)*H-Mf*GAMz*(eye(ns)-H)-MF*GAMe-MEY*LBz*H;
        rese    = (PF0-Pf0)*PHIe+PF0*(GAMz*(G+H*R-R+F*C))+PF1*GAMe+PY0*(LBX*OMe+(LBz*(G+H*R)+LBs)+(LBz*F+LBxi)*C)-Py0*LBz*(F*B+G+H*R-R)+Ps;
        ress    = (PF0-Pf0)*PHIs+PF0*(GAMz*((G+H*R-R)*R+F*B))+PF1*GAMs+PY0*(LBX*OMs+(LBz*(G+H*R)+LBs)*R+(LBz*F+LBxi)*B)-Py0*LBz*(F*B+(G+H*R-R)*R)+PY1*LBs+Ps*R;
        disp(resx)
        disp(resX)
        disp(resz)
        disp(reszb)
        disp(resxi)
        disp(reslb)
        disp(rese)
        disp(ress)
        switch exoxi_
            case 1
                reset   = (PF0-Pf0)*PHIet+PF0*GAMz*F+PF1*GAMet+PY0*(LBX*OMet+LBxi)+(PY0-Py0)*LBz*F+PY1*LBet;
                disp(reset)
        end
end
