function sol=solve_model(model)

Myy     = model.myy;
Myx     = model.myx;
Myz     = model.myz;
Mxx0    = model.mxx0;
Mxx1    = model.mxx1;
Mxz0    = model.mxz0;
Mxz1    = model.mxz1;
Mxy0    = model.mxy0;
Mxy1    = model.mxy1;
Me      = model.me;
PHI     = model.phi;
SIG     = model.sig;
nf      = model.forward;
nx      = model.backward;
ns      = size(Me,1);
ny      = size(Myy,2);

if isfield(model,'info');
    INFO=model.info;
    if any(any(INFO==0))
        Myz = [Myz zeros(ny,ns)];
        Mxz0= [Mxz0 zeros(nx+nf,ns)];
        Mxz1= [Mxz1 zeros(nx+nf,ns)];
        PHI = [PHI zeros(ns);eye(ns) zeros(ns)];
        Me  = [Me;zeros(ns)];

        R1  = [];
        R2  = [];
        is1 = [];
        for i=1:ns;
            itmp= find(INFO(:,i));
            n1  = sum(itmp);
            R1t = zeros(nf,nf*2*ns);
            R2t = zeros(nf,nf*2*ns);
            R20 = zeros(nf,nf*2*ns);
            R21 = zeros(nf,nf*2*ns);
            if n1==nf
                R1t(:,(i-1)*nf+1:i*nf)          = eye(nf);
                R2t(:,nf*ns+((i-1)*nf+1:i*nf))  = eye(nf);
                is1                             = [is1;INFO(:,i)];
            elseif n1==0
                R1t                             = [];
                R2t(:,1:ns*nf)                  = kron(PHI(i,:),eye(nf));
                R2t(:,nf*ns+((i-1)*nf+1:i*nf))  = eye(nf);
                is1                             = [is1;INFO(:,i)];
            else
                R1t(:,(i-1)*nf+1:i*nf)          = eye(nf);
                R1t                             = R1t(itmp,:);
                R20(:,nf*ns+((i-1)*nf+1:i*nf))  = eye(nf);
                R21(:,1:2*ns*nf)                = kron(PHI(i,:),eye(nf));
                R2t                             = R20+repmat(1-INFO(:,i),1,nf*2*ns).*R21;
                is1                             = [is1;INFO(:,i)];
            end
            R1 = [R1;R1t];
            R2 = [R2;R2t];
        end;
        R   = [R1;R2];
        is1 = [is1;ones(nf*ns,1)];
        ind = find(is1);
        ns  = 2*ns;
    else
        PHI = PHI;
        R   = 1;
        ind = 1:ns*nf;
    end
else
    PHI = PHI;
    R   = 1;
    ind = 1:ns*nf;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                         SOLVING THE MODEL                                   %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Mx          = Myy\Myx;
Mz          = Myy\Myz;
M0          = Mxx0-Mxy0*Mx;
M1          =-(Mxx1-Mxy1*Mx);
M2          = (Mxz0+Mxy0*Mz)*PHI+Mxz1+Mxy1*Mz;

[s,t,q,z]   = qz(M0,M1);            % Generalized Eigenvalues
[s,t,q,z]   = ordqz(s,t,q,z,'udo'); % Reorder eigenvalues and eigenvectors
z21         = z(nx+1:end,1:nx);
z11         = z(1:nx,1:nx);

if rank(z11)<nx;
    warning('Invertibility condition violated')
    sol.mx  = [];
    sol.me  = [];
    sol.my  = [];
    sol.sig = [];
    sol.id  = -1;
else
    z11i    = z11\eye(nx);
    s11     = s(1:nx,1:nx);
    t11     = t(1:nx,1:nx);

    if or(abs(t(nx,nx))>abs(s(nx,nx)),abs(t(nx+1,nx+1))<abs(s(nx+1,nx+1)));
        for i=1:size(s,1);
            lb  = t(i,i)/s(i,i);
%             if ~isreal(lb)
%                 if imag(lb)<0
%                     fprintf('lambda(%d)=%g-%gi \n',i,real(lb),abs(imag(lb)))
%                 else
%                     fprintf('lambda(%d)=%g+%gi \n',i,real(lb),imag(lb))
%                 end
%             else
%                 fprintf('lambda(%d)=%g \n',i,lb)
%             end
        end
%         warning('MATLAB:NotSaddlePath','Wrong number of stable eigenvalues.');
        sol.mx  = [];
        sol.me  = [];
        sol.my  = [];
        sol.sig = [];
        sol.id  = 1;
    else

        dyn     = s11\t11;
        FX      = z21*z11i;
        if max(max(imag(FX)))<1e-6;
            FX  = real(FX);
        end
        MX      = z11*dyn*z11i;
        if max(max(imag(MX)))<1e-6;
            MX  = real(MX);
        end
        M       = [eye(nx);FX];
        PX      = Mx*M;

        B       = 1:nx;
        F       = nx+1:nx+nf;

        PSI0    =-(M0(B,B)+M0(B,F)*FX)\M0(B,F);
        PSI1    = (M0(B,B)+M0(B,F)*FX)\M1(B,F);
        PSI2    = (M0(B,B)+M0(B,F)*FX)\M2(B,:);
        G0      = (M0(F,B)+M0(F,F)*FX)*PSI0+M0(F,F);
        G1      = (M0(F,B)+M0(F,F)*FX)*PSI1-M1(F,F);
        G2      = M2(F,:)-(M0(F,B)+M0(F,F)*FX)*PSI2;

        Q       = (R*(kron(PHI',G0)+kron(eye(ns),G1)));
        Q       = Q(:,ind);
        tmp     = Q\(R*G2(:));
        FZ      = zeros(nf*ns,1);
        FZ(ind) = tmp;
        FZ      = reshape(FZ,nf,ns);
        MZ      = PSI0*FZ*PHI+PSI1*FZ+PSI2;
        PZ      = Mx*[zeros(nx,ns);FZ]+Mz;

        sol.mx  = [MX MZ;zeros(ns,nx) PHI];
        sol.me  = [zeros(nx,size(Me,2));Me];
        sol.my  = [PX PZ];
        sol.sig = SIG;
        sol.id  = 0;
    end
end
