addpath('method');
addpath('models');
rho         = 0.8;
K           = 12;
nchain      = 2;
NDraws      = 200000;
step        = 20;
spmodel     = 'nk_model_utilization_ci_timing';
fpmodel     = 'real_model_utilization_ci_timing';
wmin        = 2*pi/32;
wmax        = 2*pi/6;
%% Sticky Price
filemetro   = 'results/metro_nk_ci_timing_';
TH          = zeros(nchain*NDraws,25);
for i=1:nchain
    eval(['load ' filemetro int2str(i) '.mat'])
    TH((i-1)*NDraws+1:i*NDraws,:)   = theta;
end
mTH = median(TH(1:step:end,:));
nth = size(TH,1);
sels= [1:4 6:7];
sels= [1 4 6 7 1 1 1];
nsel= length(sels);
sol         = feval(spmodel,mTH);
[VD,CY,CH,CI] = vdecfreq(sol,wmin,wmax,sels);
ne      = 10;
mvdsp   = reshape(VD(:),nsel,ne)';
mcysp   = reshape(CY(:),nsel,ne)';
mcisp   = reshape(CI(:),nsel,ne)';
mchsp   = reshape(CH(:),nsel,ne)';

%% Flexible Price
filemetro   = 'results/metro_real_util_ci';
TH          = zeros(nchain*NDraws,19);
for i=1:nchain
    eval(['load ' filemetro int2str(i) '.mat'])
    TH((i-1)*NDraws+1:i*NDraws,:)   = theta;
end
mTH  = median(TH(1:step:end,:));
nth = size(TH,1);
sels= 1:4;
nsel= length(sels);
sol = feval(fpmodel,mTH);
[VD,CY,CH,CI] = vdecfreq(sol,wmin,wmax,sels);
mvdfp   = reshape(VD(:),nsel,8)';
mcyfp   = reshape(CY(:),nsel,8)';
mcifp   = reshape(CI(:),nsel,8)';
mchfp   = reshape(CH(:),nsel,8)';

%% Table
str={'Ap','An','At','Ip','It','C','G','MP','Xi'};
for i=1:9
    fprintf('%s \n',str{i})
    if i<=7
        fprintf('Flexible Price   & %6.2f & %6.2f & %6.2f & %6.2f & -- & -- \\\\ \n',mvdfp(i,:))
    elseif i==9
        fprintf('Flexible Price   & %6.2f & %6.2f & %6.2f & %6.2f & -- & -- \\\\ \n',mvdfp(8,:))
    else
        fprintf('Flexible Price   & -- & -- & -- & -- & -- & -- \\\\ \n',mvdfp(8,:))
    end
    fprintf('Sticky Price     & %6.2f & %6.2f & %6.2f & %6.2f & %6.2f & %6.2f \\\\ \n',mvdsp(i,:))
end
disp(' ')
for i=1:9
    fprintf('%s \n',str{i})
    if i==8
        fprintf('Flexible Price   & -- & -- & -- & -- & -- & -- \\\\ \n',mvdfp(8,:))
    elseif i==9
        X=100*[mcyfp(8,[4 3 2]) mchfp(8,[3 2]) mcifp(8,2)];
        fprintf('Flexible Price   & %6.2f & %6.2f & %6.2f & %6.2f & %6.2f & %6.2f \\\\ \n',X)
    else
        X=100*[mcyfp(i,[4 3 2]) mchfp(i,[3 2]) mcifp(i,2)];
        fprintf('Flexible Price   & %6.2f & %6.2f & %6.2f & %6.2f & %6.2f & %6.2f \\\\ \n',X)
    end
    X=100*[mcysp(i,[4 3 2]) mchsp(i,[3 2]) mcisp(i,2)];
    fprintf('Sticky Price     & %6.2f & %6.2f & %6.2f & %6.2f & %6.2f & %6.2f \\\\ \n',X)
end
